<div align="center">
<img src="RMDB.jpg"  width=25%  /> 
</div>

# 开发指南

## 开发环境

使用 docker 

```bash
docker pull witchelaina/rmdb:latest
docker run -d -it --name rmdb --mount type=bind,source="$(pwd)",target=/root/rmdb witchelaina/rmdb:latest
```

```bash
docker exec -it rmdb /bin/bash
```


# 开发规范

## Git 使用

### commit message 规范

- `feat(ex-*):` 新功能，如 `feat(ex-1):新增查询功能` 表示第一题新增查询功能
- `fix(ex-*):` 修复问题/BUG， 如 `fix(ex-1):修复查询功能的一个 BUG` 表示修复第一题查询功能的 BUG
- `docs(ex-*):` 文档，如 `docs(ex-1):更新 README.md` 表示更新第一题的 README.md 文档
- `add:` 添加依赖等必要文件。
- `other:` 其他修改。


## Git 分支

### 主要分支

- `master` 分支为主分支，用于发布稳定版本，只能从 `ex-*` 合并，不允许其他操作。
- `ex-*` 分支为开发分支，每个新题目会创建一个新的分支，如第一题分支为`ex-1`，第一题开发完成后，通过测试需要将其合并到`master`分支。
- `test` 分支为测试分支，用于在测试环境中进行代码测试与验收。

### 个人可修改分支

- `dev-*` 分支为功能开发分支，每个人在开发时，从`master`分支创建自己的分支，如`dev-query`，在自己的分支上进行开发
- `fix-*` 分支为 bug 修复分支，每个人在开发时，从`master`分支创建自己的分支，如`fix-query`，在自己的分支上进行 bug 修复

# 开发配置

首先确保正确安装环境，随后在项目根目录下

```bash
# 给予脚本执行权限，只用执行一次
chmod +x ./scripts/*.sh

# 构建服务端
./scripts/build_server.sh

# 构建客户端
./scripts/build_client.sh
```

运行服务端

```bash
cd build/rmdb
./bin/rmdb
```

运行客户端

```bash
cd build/rmdb_client
./rmdb_client
```


---

# RMDB

全国大学生计算机系统能力大赛数据库管理系统赛道，以培养学生“数据库管理系统内核实现”能力为目标。本次比赛为参赛队伍提供数据库管理系统代码框架RMDB，参赛队伍在RMDB的基础上，设计和实现一个完整的关系型数据库管理系统，该系统要求具备运行TPC-C基准测试（TPC-C是一个面向联机事务处理的测试基准）常用负载的能力。

RMDB由中国人民大学数据库教学团队开发，平台、赛题和测试用例等得到了全国大学生计算机系统能力大赛数据库管理系统赛道技术委员会的支持和审核。系统能力大赛专家组和[101计划数据库系统课程](http://101.pku.edu.cn/courseDetails?id=DC767C683D697417E0555943CA7634DE)工作组给予了指导。

## 实验环境：
- 操作系统：Ubuntu 18.04 及以上(64位)
- 编译器：GCC
- 编程语言：C++17
- 管理工具：cmake
- 推荐编辑器：VScode

### 依赖环境库配置：
- gcc 7.1及以上版本（要求完全支持C++17）
- cmake 3.16及以上版本
- flex
- bison
- readline

欲查看有关依赖运行库和编译工具的更多信息，以及如何运行的说明，请查阅[RMDB使用文档](RMDB使用文档.pdf)

欲了解如何在非Linux系统PC上部署实验环境的指导，请查阅[RMDB环境配置文档](RMDB环境配置文档.pdf)

### 项目说明文档

- [RMDB环境配置文档](RMDB环境配置文档.pdf)
- [RMDB使用文档](RMDB使用文档.pdf)
- [RMDB项目结构](RMDB项目结构.pdf)
- [测试说明文档](测试说明文档.pdf)

## 推荐参考资料

- [**Database System Concepts** (***Seventh Edition***)](https://db-book.com/)
- [PostgreSQL 数据库内核分析](https://book.douban.com/subject/6971366//)
- [数据库系统实现](https://book.douban.com/subject/4838430/)
- [数据库系统概论(第5版)](http://chinadb.ruc.edu.cn/second/url/2)

## License
RMDB采用[木兰宽松许可证，第2版](https://license.coscl.org.cn/MulanPSL2)，可以自由拷贝和使用源码, 当做修改或分发时, 请遵守[木兰宽松许可证，第2版](https://license.coscl.org.cn/MulanPSL2).